# -*- coding: utf-8 -*-

"""Main module Solaris """
from __future__ import absolute_import
from __future__ import unicode_literals

import os
import logging
from datetime import date
from pathlib import Path
from collections import OrderedDict
import pdfrw, pdfrw.errors, pdfrw.objects, pdfrw.objects.pdfname

from ngoschema.protocols import with_metaclass, SchemaMetaclass, ObjectProtocol

from reportlab.platypus import PageTemplate, BaseDocTemplate, SimpleDocTemplate, Frame
from reportlab.platypus import NextPageTemplate, Paragraph, PageBreak, Spacer
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.pdfbase.pdfmetrics import stringWidth
from reportlab.lib.pagesizes import A4
from reportlab.lib.enums import TA_JUSTIFY

from .. import settings
from .pdfrw_utils import get_form_fields, create_blank_page
from .pdfrw_utils import ANNOT_KEY, SUBTYPE_KEY, WIDGET_SUBTYPE_KEY, ANNOT_FIELD_KEY, ANNOT_PARENT_KEY
from .personas import Member, Administrator, Coordinator

ROOT_DIR = Path(__file__).parent.parent

FORM_PAGE = Path(settings.FORM_PAGE)
FORM_PAGE = str(FORM_PAGE.resolve()) if FORM_PAGE.exists()\
    else str(ROOT_DIR.joinpath(settings.FORM_PAGE).resolve())


class Cell(with_metaclass(SchemaMetaclass)):
    """Cellule SOLARIS

    Permet le parametrage de la cellule (identifiant, email),
    le parametrage du repertoire de travail contenant les fiches des membres,
    le chargement des fiches des membres en memoire.

    :param cell_id: identifiant de cellule.
    :param email: protonmail de la cellule.
    :param cell_dir: repertoire de travail de la cellule.
    """
    _id = r"https://solaris-france.org/nono#/$defs/cells/$defs/Cell"
    _useContext = True

    def __init__(self, _value=None, **opts):
        """Initialise la cellule."""
        ObjectProtocol.__init__(self, value=_value, **opts)

    def view(self):
        """Affiche les infos de la cellule."""
        ret = ['{']
        for k, v in self.do_serialize().items():
            ret.append(f'\t{k}: {v}')
        ret.append('}')
        return '\n'.join(ret)

    def _read_members(self, member_dir):
        members = []
        cyear = date.today().year
        for x in os.listdir(str(member_dir)):
            if x.endswith(".pdf"):
                self._logger.info('LOAD FILE %s.' % x)
                pdf_fp = member_dir.joinpath(x)
                pdfr = pdfrw.PdfReader(str(pdf_fp))
                member = get_form_fields(pdfr.pages[0])
                if not member.get('GenderField'):
                    # establish gender
                    gender = 'M' if member['MaleCheck'] else ''
                    gender += 'F' if member['FemaleCheck'] else ''
                    # read fields, set gender, compute age and set title
                    member['GenderField'] = gender
                elif 'M' in member['GenderField']:
                    member['MaleCheck'] = True
                elif 'F' in member['GenderField']:
                    member['FemaleCheck'] = True
                firstn = member['FirstNameField']
                lastn = member.get('NameField') or member.get('LastNameField')
                member['LastNameField'] = lastn
                lastn_upper = lastn.upper()
                city = member['CityField']
                city_upper = member['CityUpper'] = city.upper()
                age_suffix = ''
                if member.get('BirthYearField'):
                    byear = int(member['BirthYearField'].replace('.', '/').split('/')[-1])
                    if byear < 100:
                        byear += 1900
                    member['BirthYearField'] = str(byear)
                    age = cyear - byear
                    age_suffix = f'{age}ans'
                member['TitleField'] = f'{firstn} {lastn_upper}, {city_upper}, {gender} {age_suffix}'
                member['IndexEntryField'] = f'{lastn_upper} {firstn}, {city_upper}, {gender} {age_suffix}'
                member['IndexCityEntryField'] = f'{lastn_upper} {firstn}, {gender} {age_suffix}'
                member['PageFilename'] = f'{city_upper} - {lastn_upper} {firstn}.pdf'
                member['FormOrigFilepath'] = pdf_fp
                member['EmailLink'] = f'mailto:{member["EmailField"]}'
                member['TelegramIdLink'] = f'https://t.me/{member["TelegramIdField"]}'
                members.append(member)
        return members

    def _sort_members(self):
        self.members = members = sorted(self.members, key=lambda x: x['IndexEntryField'])
        cities = sorted(list(set([m['CityField'] for m in members])))
        self.members_city = OrderedDict([(c, list()) for c in cities])
        for i, m in enumerate(self.members):
            self.members_city[m['CityField']].append(m)
            m['PageField'] = ip = str(i + 1)

    def load_members(self, member_dir=None):
        """Charge les fiches des membres de la cellule en mémoire.

        :param member_dir: repertoire contenant les fiches des membres
        :return: self
        """
        member_dir = member_dir or self.member_dir
        self.members += self._read_members(member_dir)
        self._sort_members()
        return self

    def write_form(self):
        """Crée une copie locale du formulaire"""
        import shutil
        form_fp1 = FORM_PAGE
        form_fp2 = self.cell_dir.joinpath(form_fp1.name).resolve()
        shutil.copy(str(form_fp1), str(form_fp2))
        self._logger.info('CREATE FILE %s.' % form_fp2)
        return form_fp2



# -*- coding: utf-8 -*-

"""Personas module Solaris """
from __future__ import absolute_import
from __future__ import unicode_literals

import os
import logging
from datetime import date
from pathlib import Path
from collections import OrderedDict
import pdfrw, pdfrw.errors, pdfrw.objects, pdfrw.objects.pdfname

from ngoschema.protocols import with_metaclass, SchemaMetaclass
#from vishuda.models import User


class Member(with_metaclass(SchemaMetaclass)):
    _id = r"https://solaris-france.org/nono#/$defs/personas/$defs/Member"


class Coordinator(with_metaclass(SchemaMetaclass)):
    _id = r"https://solaris-france.org/nono#/$defs/personas/$defs/Coordinator"


class Administrator(with_metaclass(SchemaMetaclass)):
    _id = r"https://solaris-france.org/nono#/$defs/personas/$defs/Administrator"




# -*- coding: utf-8 -*-

"""Main module Solaris """
from __future__ import absolute_import
from __future__ import unicode_literals

import pdfrw, pdfrw.errors, pdfrw.objects, pdfrw.objects.pdfname


# PDF ANNOTATIONS
ANNOT_KEY = '/Annots'
ANNOT_FIELD_KEY = '/T'
ANNOT_PARENT_KEY = '/Parent'
ANNOT_VAL_KEY = '/V'
ANNOT_RECT_KEY = '/Rect'
SUBTYPE_KEY = '/Subtype'
WIDGET_SUBTYPE_KEY = '/Widget'


def create_blank_page(width=None, height=None):
    blank = pdfrw.PageMerge()
    blank.mbox = [0, 0, width or 595, height or 773]
    return blank.render()


def get_form_fields(page):
    fields = {}
    annotations = page[ANNOT_KEY]
    for annotation in annotations:
        if annotation[SUBTYPE_KEY] == WIDGET_SUBTYPE_KEY:
            if not annotation[ANNOT_FIELD_KEY]:
                annotation = annotation[ANNOT_PARENT_KEY]
            if annotation[ANNOT_FIELD_KEY]:
                key = annotation[ANNOT_FIELD_KEY][1:-1]
                v = annotation[ANNOT_VAL_KEY]
                if isinstance(v, pdfrw.objects.PdfString):
                    fields[key] = v.to_unicode()
                if isinstance(v, pdfrw.objects.pdfname.BasePdfName):
                    if v == '/Yes':
                        fields[key] = True
                    if v == '/Off':
                        fields[key] = False
    return fields



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `nonosolaris` package."""
from click.testing import CliRunner

from nonosolaris.cli import cli

# PROTECTED REGION ID(nonosolaris.tests.test_ngosolaris) ENABLED START

def test_cli():
    runner = CliRunner()
    result = runner.invoke(cli, ['annuaire', 'init', 'compile'])
    #assert result.output == 'Hello World!\n'
    assert result.exit_code == 0

def test_cli2():
    from nonosolaris.commands import cmd_cell, cmd_address_book
    runner = CliRunner()
    #result = runner.invoke(cli, [])
    result = runner.invoke(cli, ['cell'])
    #result = runner.invoke(cli, ['cell', 'init'])
    #assert result.output == 'Hello World!\n'
    assert result.exit_code == 0

def test_ngosolaris():
    from nonosolaris import Cell, AddressBook
    # assert solaris
    cell = Cell(
        cell_id='Ma Cellule'
    )
    cell.load_members()
    addr_book = AddressBook(cell=cell)
    addr_book.write_member_updated_forms()
    addr_book.write_edition()


if __name__ == '__main__':
    # to run test file standalone
    #test_ngosolaris()
    #test_cli()
    test_cli2()

# PROTECTED REGION END



# -*- coding: utf-8 -*-

"""Top-level package for NonoSolaris."""

__author__ = """Cedric ROMAN"""
__email__ = 'roman@numengo.com'
__version__ = '1.1.11'

## PROTECTED REGION ID(nonosolaris.init) ENABLED START

from ngoschema.loaders import register_module
register_module('nonosolaris')

from ._settings import settings, APP_CONTEXT, DEFAULT_CONTEXT

__all__ = [
    'settings',
]
# PROTECTED REGION END



SIMPLE_SETTINGS = {
    'OVERRIDE_BY_ENV': True,
    'CONFIGURE_LOGGING': True,
    'DYNAMIC_SETTINGS': {
        'backend': 'redis',
        'pattern': 'DYNAMIC_*',
        'auto_casting': True,
        'prefix': 'NONOSOLARIS_'
    }
}

DEFAULT_CONTEXT = {
    'cell_id': 'Ma Cellule Solaris',
    'cell': {
    }
}

# config settings
CLI_CONTEXT_FILENAME = 'solaris.nono'
SOLARIS_DIR = '~/Solaris'

# directories
MEMBER_DIRNAME = 'fiches'
BUILD_DIRNAME = 'build'
PAGES_DIRNAME = 'pages'
FORMS_DIRNAME = 'forms'

# templates
FORM_PAGE = 'templates/formulaire_annuaire_v1.0.pdf'
COVER_PAGE = 'templates/cover_annuaire_v1.0.pdf'
ADDR_BOOK_PAGE = 'templates/page_annuaire_v1.0.pdf'
ADDR_BOOK_PREFIX = 'annuaire-solaris'

# layout
INDEX_TITLE_SPACE = 40
TOC_SPACE = 25
INDEX_MARGIN = 20



# *- coding: utf-8 -*-
"""click API for AddressBook as 'AddressBook'."""

import click
import gettext
import os

from ngoschema.cli import pass_environment, ComplexGroup


#### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.imports) ENABLED START
from .. import DEFAULT_CONTEXT, APP_CONTEXT
from nonosolaris.models.address_book import AddressBook

# loads locally defined context file if in execution folder
DEFAULT_CONTEXT.load_default_context(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)

addressbook = AddressBook()
# set environment variables to default object values (then used as default values in CLI)
os.environ.setdefault('NONOSOLARIS_ADDRESSBOOK_EDITION', addressbook.items_serialize('edition'))
os.environ.setdefault('NONOSOLARIS_ADDRESSBOOK_EDITION_FMT', addressbook.items_serialize('edition_fmt'))
#### PROTECTED REGION END
_ = gettext.gettext


@click.group('address-book', cls=ComplexGroup, help=ComplexGroup.format_docstring(AddressBook.__doc__, arguments=True), chain=True)
@pass_environment
def cli(ctx):
    __doc__ = AddressBook.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.cli) ENABLED START
    # load context file in home directory for possibly new default values
    cfg = ctx.resolve_path(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)
    ctx.load_context_file(cfg)
    ctx.rc.add_local_entries(**ctx.rc.get('addressbook', {}))
    ctx.addressbook = ctx.obj = AddressBook(context=ctx.rc)
    # update environment variables with possibly new defaults
    os.environ.setdefault('NONOSOLARIS_ADDRESSBOOK_EDITION_FMT', addressbook.items_serialize('edition_fmt'))
    os.environ.setdefault('NONOSOLARIS_ADDRESSBOOK_EDITION', addressbook.items_serialize('edition'))
    # add current object to local context
    ctx.rc.add_local_entry('addressbook', ctx.addressbook)
    #### PROTECTED REGION END


@cli.command('init', help=ComplexGroup.format_docstring(AddressBook.__init__.__doc__))
@click.option('--edition', default=lambda: os.environ.get("NONOSOLARIS_ADDRESSBOOK_EDITION"))
@click.option('--edition-fmt', default=lambda: os.environ.get("NONOSOLARIS_ADDRESSBOOK_EDITION_FMT"))
@pass_environment
def init(ctx, edition, edition_fmt):
    __doc__ = AddressBook.__init__.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.init) ENABLED START
    click.echo('CALL init')
    ret = ctx.addressbook.__init__(edition=edition, edition_fmt=edition_fmt)
    # update environment variables
    os.environ['NONOSOLARIS_ADDRESSBOOK_EDITION'] = ctx.addressbook.items_serialize('edition')
    os.environ['NONOSOLARIS_ADDRESSBOOK_EDITION_FMT'] = ctx.addressbook.items_serialize('edition_fmt')
    # save context
    cfg = ctx.resolve_path(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)
    ctx.save_context_file(cfg)
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END


@cli.command('write-edition', help=ComplexGroup.format_docstring(AddressBook.write_edition.__doc__))
@pass_environment
def write_edition(ctx):
    __doc__ = AddressBook.write_edition.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.write_edition) ENABLED START
    click.echo('CALL write-edition')
    ret = ctx.addressbook.write_edition()
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END


@cli.command('write-member-updated-forms', help=ComplexGroup.format_docstring(AddressBook.write_member_updated_forms.__doc__))
@pass_environment
def write_member_updated_forms(ctx):
    __doc__ = AddressBook.write_member_updated_forms.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.write_member_updated_forms) ENABLED START
    click.echo('CALL write-member-updated-forms')
    ret = ctx.addressbook.write_member_updated_forms()
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END



#### PROTECTED REGION ID(nonosolaris.commands.cmd_address_book.user) ENABLED START ####
# Your stuff...

#### PROTECTED REGION END ####



from simple_settings import LazySettings
from ngoschema.config.utils import search_app_config_files

from ngoschema import settings, APP_CONTEXT, DEFAULT_CONTEXT
from nonosolaris.config import settings as local_settings

settings = LazySettings('ngoschema.config.settings',
                        'nonosolaris.config.settings',
                        *search_app_config_files('NonoSolaris', 'solaris'),
                        )

DEFAULT_CONTEXT.add_local_entries(settings=settings, **getattr(settings, 'DEFAULT_CONTEXT', {}))
APP_CONTEXT.add_local_entries(_nonosolaris_env=settings.as_dict())



# *- coding: utf-8 -*-
"""click API for Cell as 'Cell'."""

import click
import gettext
import os

from ngoschema.cli import pass_environment, ComplexGroup


#### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.imports) ENABLED START
from .. import DEFAULT_CONTEXT, APP_CONTEXT
from nonosolaris.models.cells import Cell

# loads locally defined context file if in execution folder
DEFAULT_CONTEXT.load_default_context(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)

cell = Cell()
# set environment variables to default object values (then used as default values in CLI)
os.environ.setdefault('NONOSOLARIS_CELL_CELL_DIR', cell.items_serialize('cell_dir'))
os.environ.setdefault('NONOSOLARIS_CELL_CELL_ID', cell.items_serialize('cell_id'))
#### PROTECTED REGION END
_ = gettext.gettext


@click.group('cell', cls=ComplexGroup, help=ComplexGroup.format_docstring(Cell.__doc__, arguments=True), chain=True)
@pass_environment
def cli(ctx):
    __doc__ = Cell.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.cli) ENABLED START
    # load context file in home directory for possibly new default values
    cfg = ctx.resolve_path(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)
    ctx.load_context_file(cfg)
    ctx.rc.add_local_entries(**ctx.rc.get('cell', {}))
    ctx.cell = ctx.obj = Cell(context=ctx.rc)
    # update environment variables with possibly new defaults
    os.environ.setdefault('NONOSOLARIS_CELL_CELL_ID', cell.items_serialize('cell_id'))
    os.environ.setdefault('NONOSOLARIS_CELL_CELL_DIR', cell.items_serialize('cell_dir'))
    # add current object to local context
    ctx.rc.add_local_entry('cell', ctx.cell)
    #### PROTECTED REGION END


@cli.command('init', help=ComplexGroup.format_docstring(Cell.__init__.__doc__))
@click.option('--cell-id', default=lambda: os.environ.get("NONOSOLARIS_CELL_CELL_ID"), prompt=True, prompt_required=True)
@click.option('--cell-dir', default=lambda: os.environ.get("NONOSOLARIS_CELL_CELL_DIR"), type=click.Path())
@click.option('--email')
@click.option('--telegram-channel')
@pass_environment
def init(ctx, cell_id, cell_dir, email, telegram_channel):
    __doc__ = Cell.__init__.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.init) ENABLED START
    cell_dir = ctx.resolve_path(cell_dir)
    click.echo('CALL init')
    ret = ctx.cell.__init__(cell_id=cell_id, cell_dir=cell_dir, email=email, telegram_channel=telegram_channel)
    # update environment variables
    os.environ['NONOSOLARIS_CELL_CELL_ID'] = ctx.cell.items_serialize('cell_id')
    os.environ['NONOSOLARIS_CELL_CELL_DIR'] = ctx.cell.items_serialize('cell_dir')
    # save context
    cfg = ctx.resolve_path(APP_CONTEXT['settings'].CLI_CONTEXT_FILENAME)
    ctx.save_context_file(cfg)
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END


@cli.command('view', help=ComplexGroup.format_docstring(Cell.view.__doc__))
@pass_environment
def view(ctx):
    __doc__ = Cell.view.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.view) ENABLED START
    click.echo('CALL view')
    ret = ctx.cell.view()
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END


@cli.command('load-members', help=ComplexGroup.format_docstring(Cell.load_members.__doc__))
@click.option('--member-dir', default=None)
@pass_environment
def load_members(ctx, member_dir):
    __doc__ = Cell.load_members.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.load_members) ENABLED START
    click.echo('CALL load-members')
    ret = ctx.cell.load_members(member_dir=member_dir)
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END


@cli.command('write-form', help=ComplexGroup.format_docstring(Cell.write_form.__doc__))
@pass_environment
def write_form(ctx):
    __doc__ = Cell.write_form.__doc__
    #### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.write_form) ENABLED START
    click.echo('CALL write-form')
    ret = ctx.cell.write_form()
    if ret:
        click.echo(str(ret))
    #### PROTECTED REGION END



#### PROTECTED REGION ID(nonosolaris.commands.cmd_cell.user) ENABLED START ####
# Your stuff...

#### PROTECTED REGION END ####



# -*- coding: utf-8 -*-

"""Main module Address Book """
from __future__ import absolute_import
from __future__ import unicode_literals

import os
import logging
from datetime import date
from pathlib import Path
from collections import OrderedDict
import pdfrw, pdfrw.errors, pdfrw.objects, pdfrw.objects.pdfname

from ngoschema.protocols import with_metaclass, SchemaMetaclass, ObjectProtocol

from reportlab.platypus import SimpleDocTemplate
from reportlab.platypus import Paragraph, PageBreak, Spacer
from reportlab.lib.styles import ParagraphStyle
from reportlab.pdfbase.pdfmetrics import stringWidth
from reportlab.lib.pagesizes import A4
from reportlab.lib.enums import TA_JUSTIFY

from .. import settings
from .pdfrw_utils import create_blank_page
from .pdfrw_utils import ANNOT_KEY, SUBTYPE_KEY, WIDGET_SUBTYPE_KEY, ANNOT_FIELD_KEY, ANNOT_PARENT_KEY
from .cells import Cell

ROOT_DIR = Path(__file__).parent.parent

FORM_PAGE = Path(settings.FORM_PAGE)
COVER_PAGE = Path(settings.COVER_PAGE)
ADDR_BOOK_PAGE = Path(settings.ADDR_BOOK_PAGE)

FORM_PAGE = str(FORM_PAGE.resolve()) if FORM_PAGE.exists()\
    else str(ROOT_DIR.joinpath(settings.FORM_PAGE).resolve())
COVER_PAGE = str(COVER_PAGE.resolve()) if COVER_PAGE.exists()\
    else str(ROOT_DIR.joinpath(settings.COVER_PAGE).resolve())
ADDR_BOOK_PAGE = str(ADDR_BOOK_PAGE.resolve()) if ADDR_BOOK_PAGE.exists()\
    else str(ROOT_DIR.joinpath(settings.ADDR_BOOK_PAGE).resolve())


class AddressBook(with_metaclass(SchemaMetaclass)):
    """Annuaire SOLARIS

    Permet l'édition des annuaires d une cellule. L'annuaire s initialise
    à partir d une cellule dont les fiches des membres ont été chargés.
    L'annuaire permet de compiler un annuaire indexé à partir des fiches individuelles des membres
    en gardant une tracabilité des versions.
    L'annuaire permet également de mettre à jour les fiches au dernier format.
    """
    _id = r"https://solaris-france.org/nono#/$defs/AddressBook"
    _useContext = True

    def __init__(self, **kwargs):
        """Initialise l annuaire et crée les répertoires de sortie."""
        ObjectProtocol.__init__(self, **kwargs)
        cell = self.cell
        # if cell is defined but members not loaded, time to do it
        if cell and not cell.members:
            cell.load_members()
        build_dir = self.cell.build_dir
        if not build_dir.exists():
            os.makedirs(str(build_dir))
            self._logger.info('CREATE DIRECTORY %s.' % build_dir)

    def _write_member_pages(self):
        pages_dir = self.build_ed_dir.joinpath(settings.PAGES_DIRNAME)
        if not pages_dir.exists():
            os.makedirs(str(pages_dir))
            self._logger.info('CREATE DIRECTORY %s.' % pages_dir)
        cell_id = self.cell.cell_id
        for member in self.cell.members:
            member_page = pdfrw.PdfReader(ADDR_BOOK_PAGE)
            mfp = pages_dir.joinpath(member['PageFilename'])
            member['PageFilepath'] = mfp
            member['EditionFilenameField'] = self.edition_fp.stem
            member_writer = pdfrw.PdfWriter(str(mfp))
            annotations = member_page.pages[0][ANNOT_KEY]
            for annotation in annotations:
                if annotation[SUBTYPE_KEY] == WIDGET_SUBTYPE_KEY:
                    if annotation[ANNOT_FIELD_KEY]:
                        key = annotation[ANNOT_FIELD_KEY][1:-1]
                        new_key = key + member['PageField']
                        if key in member.keys():
                            if type(member[key]) == bool:
                                if member[key] == True:
                                    annotation.update(pdfrw.PdfDict(AS=pdfrw.PdfName('Yes'), T=new_key))
                            else:
                                val = member[key]
                                val = pdfrw.PdfString.from_unicode(val)
                                if val:
                                    if key in ['EmailField', 'TelegramIdField']:
                                        link_key = key.replace('Field', 'Link')
                                        #annotation.AA.D.URI = pdfrw.PdfString.from_unicode(link)
                                        #annotation.AA.D.update(pdfrw.PdfDict(V=val))
                                annotation.update(pdfrw.PdfDict(V=val, T=new_key, AP=''))
                    annotation.update(pdfrw.PdfDict(Ff=1))
            member_page.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))
            member_writer.addpages(member_page.pages)
            member_writer.trailer.Info = pdfrw.IndirectPdfDict(
                Title=member['TitleField'],
                Author=f'SOLARIS {cell_id}',
                Creator=f'SOLARIS {cell_id}',
                Subject=f'Annuaire SOLARIS {cell_id} {self.edition_fmt}'
            )
            member_writer.trailer.Root.AcroForm = member_page.Root.AcroForm
            member_writer.trailer.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))
            member_writer.write()
            self._logger.info('CREATE FILE %s.' % mfp)
        return self

    def _write_index(self):
        doc = SimpleDocTemplate(str(self.index_fp),
                                pagesize=A4)

        w, h = doc.width, doc.height

        flowables = []

        header1_style = ParagraphStyle(name='IndexHeading', fontSize=18)
        header2_style = ParagraphStyle(name='IndexHeading2', fontSize=14)
        toc_entry_style = ParagraphStyle(name='toc', fontSize=12, leading=14, alignment=TA_JUSTIFY)
        fn = toc_entry_style.fontName
        fs = toc_entry_style.fontSize
        w = doc.width - 20
        dot = '.'
        dotw = stringWidth(dot, fn, fs)

        flowables.append(Paragraph('<b>Index</b>', style=header1_style))
        flowables.append(Spacer(width=0, height=settings.INDEX_TITLE_SPACE))
        for member in self.cell.members:
            ptext = member['IndexEntryField']
            pagestr = member['PageField']
            page = int(pagestr)
            availWidth = w - stringWidth(ptext+pagestr, fn, fs)
            dotsn = int(availWidth / dotw)
            toc_entry = '%s%s%s' % (ptext, dotsn * dot, pagestr)
            flowables.append(Paragraph(toc_entry, style=toc_entry_style))
        flowables.append(PageBreak())

        flowables.append(Paragraph('<b>Index par ville</b>', header1_style))
        flowables.append(Spacer(width=0, height=settings.INDEX_TITLE_SPACE))
        for city, members in self.cell.members_city.items():
            flowables.append(Spacer(width=0, height=settings.TOC_SPACE))
            flowables.append(Paragraph(f'<b>{city}</b>', style=header2_style))
            flowables.append(Spacer(width=0, height=settings.TOC_SPACE))
            for member in members:
                ptext = member['IndexCityEntryField']
                pagestr = member['PageField']
                page = int(pagestr)
                availWidth = w - stringWidth(ptext+pagestr, fn, fs)
                dotsn = int(availWidth / dotw)
                toc_entry = '%s%s%s' % (ptext, dotsn * dot, pagestr)
                flowables.append(Paragraph(toc_entry, style=toc_entry_style))

        doc.build(flowables)
        self._logger.info('CREATE FILE %s.' % self.index_fp)

    def _compile_write(self):
        cell_id = self.cell.cell_id
        all_writer = pdfrw.PdfWriter(str(self.edition_fp))
        bookmarks = []
        bookmarksDict = {}

        def addBookmark(title, pageNum, parent=None):
            '''
            Adds a new bookmark entry.
            pageNum must be a valid page number in the writer
            and parent can be a bookmark object returned by a previous addBookmark call
            '''
            try:
                page = all_writer.pagearray[pageNum]
            except IndexError:
                # TODO: Improve error handling ?
                pdfrw.errors.PdfOutputError("Invalid page number: " % (pageNum))

            bookmark = {
                'title': title,
                'page': page,
                'childs': []
            }
            bid = id(bookmark)

            if not parent:
                bookmarks.append(bookmark)

            else:
                parentObj = bookmarksDict.get(id(parent), None)
                if not parentObj:
                    pdfrw.errors.PdfOutputError("Bookmark parent object not found: " % parent)

                parentObj['childs'].append(bookmark)

            bookmarksDict[bid] = bookmark
            return bookmark

        # cover
        cover_reader = pdfrw.PdfReader(COVER_PAGE)
        cover_annotations = cover_reader.pages[0][ANNOT_KEY]
        for annotation in cover_annotations:
            if annotation[SUBTYPE_KEY] == WIDGET_SUBTYPE_KEY:
                if annotation[ANNOT_FIELD_KEY]:
                    key = annotation[ANNOT_FIELD_KEY][1:-1]
                    if key == 'CellField':
                        annotation.update(pdfrw.PdfDict(V=pdfrw.PdfString.from_unicode(cell_id), AP=''))
                    if key == 'EditionField':
                        annotation.update(pdfrw.PdfDict(V=pdfrw.PdfString.from_unicode(self.edition_fmt), AP=''))
        all_writer.addpages(cover_reader.pages)

        all_writer.addpage(create_blank_page())
        n0 = len(all_writer.pagearray)

        for member in self.cell.members:
            member_reader = pdfrw.PdfReader(ADDR_BOOK_PAGE)
            all_writer.addpages(member_reader.pages)
            annotations = all_writer.pagearray[-1][ANNOT_KEY]
            for annotation in annotations:
                if annotation[SUBTYPE_KEY] == WIDGET_SUBTYPE_KEY:
                    if not annotation[ANNOT_FIELD_KEY]:
                        annotation = annotation[ANNOT_PARENT_KEY]
                    if annotation[ANNOT_FIELD_KEY]:
                        key = annotation[ANNOT_FIELD_KEY][1:-1]
                        # create an aliased field based on page number / avoid conflict between multiple forms data
                        new_key = key + member['PageField']
                        if key in member.keys():
                            val = member[key]
                            if type(member[key]) == bool:
                                if member[key] == True:
                                    annotation.update(pdfrw.PdfDict(AS=pdfrw.PdfName('Yes')))
                            elif val:
                                val = pdfrw.PdfString.from_unicode(member[key])
                                if key in ['EmailLinkField', 'TelegramIdLinkField']:
                                    pass
                                annotation.update(pdfrw.PdfDict(V=val, T=new_key, AP=''))
                    annotation.update(pdfrw.PdfDict(Ff=1))
            real_page_num = n0+int(member['PageField'])-1
            addBookmark(member['TitleField'], real_page_num)

        all_writer.addpage(create_blank_page())
        # index
        index_reader = pdfrw.PdfReader(str(self.index_fp))
        all_writer.addpages(index_reader.pages)

        # Recursive function to build outlines tree
        # https://github.com/pmaupin/pdfrw/issues/52
        def buildOutlines(parent, bookmarks):

            outline = None

            if bookmarks:
                outline = pdfrw.IndirectPdfDict()
                outline.Count = len(bookmarks)

                first = None
                next = None
                last = None

                for b in bookmarks:

                    newb = pdfrw.IndirectPdfDict(
                        Parent=parent or outline,
                        Title=b['title'],
                        A=pdfrw.IndirectPdfDict(
                            D=pdfrw.PdfArray((b['page'], pdfrw.PdfName('Fit'))),
                            S=pdfrw.PdfName('GoTo')
                        )
                    )

                    if not first:
                        first = newb

                    else:
                        last.Next = newb
                        newb.Prev = last

                    last = newb

                    # Add children, if any.
                    if b['childs']:
                        childOutline = buildOutlines(newb, b['childs'])
                        newb.First = childOutline.First
                        newb.Last = childOutline.Last
                        newb.Count = childOutline.Count

                outline.First = first
                outline.Last = last

            return outline

        # Testing for now, only add root level bookmarks
        outlines = buildOutlines(None, bookmarks)

        if outlines:
            all_writer.trailer.Root.Outlines = outlines
        all_writer.trailer.Info = pdfrw.IndirectPdfDict(
            Title=f'Annuaire SOLARIS {cell_id} {self.edition_fmt}',
            Author=f'SOLARIS {cell_id}',
            Creator=f'SOLARIS {cell_id}'
        )
        all_writer.trailer.Root.AcroForm = cover_reader.Root.AcroForm
        all_writer.trailer.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))
        all_writer.write()
        self._logger.info('CREATE FILE %s.' % self.edition_fp)
        return self

    def write_edition(self):
        """Compile un annuaire daté et indexé."""
        build_ed_dir = self.build_ed_dir
        if not build_ed_dir.exists():
            os.makedirs(str(build_ed_dir))
            self._logger.info('CREATE DIRECTORY %s.' % build_ed_dir)
        self._write_member_pages()
        self._write_index()
        self._compile_write()
        return self.edition_fp

    def write_member_updated_forms(self):
        """Mise à jour des formulaires de membres de la cellule."""
        forms_dir = self.forms_updated_dir
        if not forms_dir.exists():
            os.makedirs(str(forms_dir))
            self._logger.info('CREATE DIRECTORY %s.' % forms_dir)
        cell_id = self.cell.cell_id
        forms = []
        for member in self.cell.members:
            member_page = pdfrw.PdfReader(FORM_PAGE)
            mfp = self.forms_updated_dir.joinpath(member['PageFilename'])
            forms.append(mfp)
            member['FormUpdatedFilepath'] = mfp
            member['EditionFilenameField'] = self.edition_fp.stem
            member_writer = pdfrw.PdfWriter(str(mfp))
            annotations = member_page.pages[0][ANNOT_KEY]
            for annotation in annotations:
                if annotation[SUBTYPE_KEY] == WIDGET_SUBTYPE_KEY:
                    if annotation[ANNOT_FIELD_KEY]:
                        key = annotation[ANNOT_FIELD_KEY][1:-1]
                        if key in member.keys():
                            if type(member[key]) == bool:
                                if member[key] == True:
                                    annotation.update(pdfrw.PdfDict(AS=pdfrw.PdfName('Yes'), T=key))
                            else:
                                val = member[key]
                                val = pdfrw.PdfString.from_unicode(val)
                                if val:
                                    if key in ['EmailField', 'TelegramIdField']:
                                        link_key = key.replace('Field', 'Link')
                                        #annotation.AA.D.URI = pdfrw.PdfString.from_unicode(link)
                                        #annotation.AA.D.update(pdfrw.PdfDict(V=val))
                                annotation.update(pdfrw.PdfDict(V=val, T=key, AP=''))
            member_page.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))
            member_writer.addpages(member_page.pages)
            member_writer.trailer.Info = pdfrw.IndirectPdfDict(
                Title=member['TitleField'],
                Author=f'SOLARIS {cell_id}',
                Creator=f'SOLARIS {cell_id}',
                Subject=f'Formulaire SOLARIS {cell_id} - {self.edition_fmt}'
            )
            member_writer.trailer.Root.AcroForm = member_page.Root.AcroForm
            member_writer.trailer.Root.AcroForm.update(pdfrw.PdfDict(NeedAppearances=pdfrw.PdfObject('true')))
            member_writer.write()
            self._logger.info('CREATE FILE %s.' % mfp)
        return forms



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

from setuptools import find_packages
from setuptools import setup
from setuptools.command.develop import develop
from setuptools.command.install import install

import io
import os
import re
import sys
import subprocess
from os.path import basename
from os.path import dirname
from os.path import join
from glob import glob
from os.path import splitext



def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8')
    ).read()


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    dir_path = dirname(os.path.realpath(__file__))
    bumpversion = open(join(dir_path, '.bumpversion.cfg')).read()
    return re.search("^current_version\s*=\s*(\S*)\s*\n",
                     bumpversion, re.MULTILINE).group(1)


name = 'nonosolaris'
package = 'nonosolaris'
description = "Nono, le petit robot d'assistance a l'entraide humaine"
url = 'https://github.com/numengo/python-nonosolaris'
author = 'Cedric ROMAN'
author_email = 'roman@numengo.com'
license = 'GNU General Public License v3'
version = get_version(package)


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(os.path.join(package))
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]
    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}

setup_requires = [
    'matrix',
    #'pytest-runner',
]

install_requires = [
    'future',
    'click',
    'ngoschema>=1.0.10',
    'reportlab',
    'pdfrw',
]

test_requires = [
    'pytest',
    'pytest-logger',
]

extras_requires = {
}

setup(
    name=name,
    version=version,
    license=license,
    description=description,
    long_description='%s\n%s' % (
        re.compile('^.. skip-next.*', re.M | re.S).
           sub('', re.compile('^.. start-badges.*^.. end-badges', re.M | re.S).
           sub('', read('README.rst'))),
        re.sub(':[a-z]+:`~?(.*?)`', r'``\1``', read('CHANGELOG.rst'))
    ),
    long_description_content_type='text/x-rst',
    author=author,
    author_email=author_email,
    url=url,
    packages=find_packages(exclude='tests'),
    package_data=get_package_data(package),
    include_package_data=True,
    zip_safe=False,
    keywords=["solaris", "nono", " contact book", " annuaire", " telegram"],
    setup_requires=setup_requires,
    install_requires=install_requires,
    tests_require=test_requires,
    extras_require=extras_requires,
    entry_points={
        'console_scripts': [
            'nonosolaris=nonosolaris.cli:cli'
        ]
    },
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
)



"""
Module that contains the command line app.

Why does this file exist, and why not put this in __main__?
  You might be tempted to import things from __main__ later, but that will cause
  problems: the code will get executed twice:
  - When you run `python -mngosolaris` python will execute
    ``__main__.py`` as a script. That means there won't be any
    ``nonosolaris.__main__`` in ``sys.modules``.
  - When you import __main__ it will get executed again (as a module) because
    there's no ``nonosolaris.__main__`` in ``sys.modules``.
Also see (1) from http://click.pocoo.org/5/setuptools/#setuptools-integration


The command line interface loads subcommands dynamically from a plugin folder and other things.

All the commands are implemented as plugins in the `nonosolaris.commands` package.
If a python module is placed named "cmd_foo" it will show up as "foo" command and the `cli` object
within it will be loaded as nested Click command.
"""
import os, sys
import click
from ngoschema.cli import ComplexCLI, base_cli, run_cli

# PROTECTED REGION ID(nonosolaris.cli) ENABLED START
# https://www.dailymotion.com/video/x3a0ef9
from pathlib import Path

from nonosolaris.ascii_art import banner_nono, banner_solaris
from nonosolaris._settings import settings, APP_CONTEXT

CONTEXT_SETTINGS = dict(auto_envvar_prefix="NONOSOLARIS", show_default=True)
CMD_FOLDER = os.path.abspath(os.path.join(os.path.dirname(__file__), "commands"))

cli = click.command(
    cls=ComplexCLI,
    name='nonosolaris',
    module_name='nonosolaris',
    banner=banner_nono,
    cmd_folder=CMD_FOLDER,
    help="Nono, le petit robot d'assistance a l entraide humaine",
    no_args_is_help=True,
    context_settings=CONTEXT_SETTINGS)(base_cli)

if __name__ == "__main__":
    # used for debug - allows to run the file and pass arguments to command line
    run_cli(cli, sys.argv[1:])

# PROTECTED REGION END
